{
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

                       SendMail for Pascal Programs

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

 ------------------------------------------------------------------------------
  SendMail Unit
 ------------------------------------------------------------------------------

  This unit contains functions for use with SendMail. Most servers or linux
  distrobutoins have SendMail installed, or can easily get it installed.
  Since this unit currently uses AssignStream, it should only work on
  Unix/Linux systems.

 ------------------------------------------------------------------------------
  Developer Notes
 ------------------------------------------------------------------------------

  [ 14/OCT/2005 -L505 ]

   -created unit

   -note: this unit is a very similar to the SendMail_Unix unit included in
          Pascal Web Unit project (PWU). At a later point in time, the two units
          will be merged so that there is less maintenance. PWU will just ship
          with it's own copy of this unit.

 ------------------------------------------------------------------------------
  Developer Todo
 ------------------------------------------------------------------------------

   - change some function paramaters to CONST which are read only, instead
     of just leaving them unspecified.
   
 ------------------------------------------------------------------------------
  Contributors/Authors:
 ------------------------------------------------------------------------------
   Lars (L505)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
}



unit SendMail;


{$DEFINE EXTRA_SECURE}

{$IFDEF FPC}{$MODE OBJFPC}{$H+}
    {$IFDEF EXTRA_SECURE}
     {$R+}{$Q+}{$CHECKPOINTER ON}
    {$ENDIF}
{$ENDIF}



{------------------------------------------------------------------------------}
interface
{------------------------------------------------------------------------------}

uses
  SendMailPrep, //mail preparation
  unix,
  sysutils;

const
  SENDMAIL_PATH ='/usr/sbin/sendmail';

type
  TEMailPriority = set of (Normal, High, Low);

  TEMailBodyType = set of (HTML, TextPlain);

  TEmailAttachment = record
    FileName: string; //name of file
    FileType: string; //content/mime type
    Disposition: set of (RegularAttach, InlineAttach); //attached separately or inline attachment
  end;

  TEmailAttachments = array of TEmailAttachment;

var
  EmailBodyType: TEmailBodyType; // public/global variable that let's the user
                            // specify the email type on the fly.
                            // i.e. html emails or plain text


{------------------------------------------------------------------------------}
{--------- PUBLIC PROCEDURE/FUNCTION DECLARATIONS -----------------------------}
{------------------------------------------------------------------------------}

  function SendMail(EmailFrom: string;
                       EmailTo: string;
                       Subject: string;
                       MessageText:string): boolean;



  function SendMail_P(Priority: TEMailPriority;
                         EmailFrom: string;
                         EmailTo: string;
                         Subject: string;
                         MessageText: string): boolean;




  function SendMail_PA(Priority: TEMailPriority;
                          AttachedFiles: TEmailAttachments;
                          EmailFrom: string;
                          EmailTo: string;
                          Subject: string;
                          MessageText: string): boolean;




//  END OF PUBLIC PROCEDURE/FUNCTION DECLARATIONS
{------------------------------------------------------------------------------}


{------------------------------------------------------------------------------}
implementation
{------------------------------------------------------------------------------}


{------------------------------------------------------------------------------}
{--------- PRIVATE PROCEDURES/FUNCTIONS ---------------------------------------}
{------------------------------------------------------------------------------}

procedure WriteContentType(var sout: text);
begin
  if EmailBodyType = [] then //default email is plain text
    writeln(sout, 'Content-Type: text/plain');
  if EmailBodyType = [HTML] then //default email is plain text
    writeln(sout, 'Content-Type: text/html');
  if EmailBodyType = [TextPlain] then //default email is plain text
    writeln(sout, 'Content-Type: text/plain');
end;


//  END OF PRIVATE PROCEDURES/FUNCTIONS
{------------------------------------------------------------------------------}



{------------------------------------------------------------------------------}
{--------- PUBLIC PROCEDURES/FUNCTIONS ----------------------------------------}
{------------------------------------------------------------------------------}


// sends an email by sendmail, with default priority (normal).
// email should be written in plain text
//
// Returns true if the mail was sent for delivery attempt, false if error
// opening sendmail. True does not indicate the mail went through, because
// it does not check for bad addresses or failed deliveries
function SendMail(EmailFrom: string;
                     EmailTo: string;
                     Subject: string;
                     MessageText:string): boolean;
var
  SIN,
  SOUT:text;
  Args: array of string;
begin
  result:= false;
  setlength(args, 1);
  Args[0]:= ' -t ';    //tells sendmail to parse for TO, FROM, CC, BCC in text

  if AssignStream(SIN, SOUT, SENDMAIL_PATH, Args) <> -1 then
  begin
    writeln(sout, 'MIME-Version: 1.0');                // header
    writeln(sout, 'X-Priority: 3 (Normal)');           //...priority normal
    writeln(sout, 'X-Mailer: SendMail for Pascal');

    WriteContentType(sout);

    writeln(sout, 'FROM: ' + EmailFrom);
    writeln(sout, 'TO: ' + EmailTo);
    writeln(sout, 'SUBJECT: ' + Subject);
    writeln(sout, MessageText);

    writeln(sout, '.'); //end connection

    result:= true;
  end else
  begin
    writeln('AssignStream returned error -1'); //ERROR REPORTING
    result:= false;
  end;

  close(sin);
  close(sout);
end;


// sends an email by sendmail, with the ability to specify priority, BCC, CCC.
// Use this you need to specify many settings of your e-mail, otherwise use the
// above functions. Email message should be written in plain text
//
// Returns true if the mail was sent for delivery attempt, false if error
// opening sendmail
function SendMail_P(Priority: TEMailPriority;
                       EmailFrom: string;
                       EmailTo: string;
                       Subject: string;
                       MessageText: string): boolean;
var
  SIN,
  SOUT:text;
  Args: array of string;
  line: string;
begin
  result:= false;
  setlength(args, 1);
  Args[0]:= '-t';    //tells sendmail to parse for TO, FROM, CC, BCC in text

  if AssignStream(SIN, SOUT, SENDMAIL_PATH, Args) <> -1 then
  begin
    writeln(sout, 'MIME-Version: 1.0');                // header
    if priority = [normal] then
      writeln(sout, 'X-Priority: 3 (Normal)');         //...priority normal
    if priority = [low] then
      writeln(sout, 'X-Priority: 5 (Low)');            //...priority low
    if priority = [high] then
      writeln(sout, 'X-Priority: 1 (High)');           //...priority high
    writeln(sout, 'X-Mailer: SendMail for Pascal');

//    writeln(sout, 'Content-Type: text/plain');
    WriteContentType(sout);

    writeln(sout, 'FROM: ' + EmailFrom);               // from
    writeln(sout, 'TO: ' + EmailTo);                   // to
    writeln(sout, 'SUBJECT: ' + Subject);              // subject
    writeln(sout);
    writeln(sout, MessageText);                        // message

    writeln(sout, '.'); //end connection
    result:= true;
  end else
  begin
    writeln('AssignStream returned error -1'); //ERROR REPORTING
    result:= false;
  end;

  close(sin);
  close(sout);

end;


// sends an email by sendmail, with the ability to specify priority and attach
// files. Use this if you need to specify the priority and attach files.
// function. Email message should be written in plain text
//
// Returns true if the mail was sent for delivery attempt, false if error
// opening sendmail
function SendMail_PA(Priority: TEMailPriority;
                        AttachedFiles: TEmailAttachments;
                        EmailFrom: string;
                        EmailTo: string;
                        Subject: string;
                        MessageText: string): boolean;
var
  SIN,
  SOUT:text;
  Args: array of string;
  TheDisposition: string;
  TheAttachedFiles: TEmailAttachments;
  iLoc: integer; //local loop integer
  BoundaryDivider: string;
begin
  result:= false;
  TheAttachedFiles:= AttachedFiles;
  setlength(args, 1);
  Args[0]:= '-t';    //tells sendmail to parse for TO, FROM, CC, BCC in text

  BoundaryDivider:= '_Divider_' + PrepEmailBoundary;

  if AssignStream(SIN, SOUT, SENDMAIL_PATH, Args) <> -1 then
  begin
    writeln(sout, 'MIME-Version: 1.0');                // header
    writeln(sout, 'FROM: ' + EmailFrom);               // from
    writeln(sout, 'TO: ' + EmailTo);                   // to
    writeln(sout, 'SUBJECT: ' + Subject);              // subject
    if priority = [normal] then
      writeln(sout, 'X-Priority: 3 (Normal)');         //...priority normal
    if priority = [low] then
      writeln(sout, 'X-Priority: 5 (Low)');            //...priority low
    if priority = [high] then
      writeln(sout, 'X-Priority: 1 (High)');           //...priority high
    writeln(sout, 'X-Mailer: SendMail for Pascal');
    writeln(sout, 'Content-Type: multipart/mixed;');
    writeln(sout, ' boundary="----=' + BoundaryDivider + '"');

    writeln(sout);
    writeln(sout, 'This is a multi-part message in MIME format.');
    writeln(sout);

    // message body text : write it
    writeln(sout, '------=' + BoundaryDivider);
//    writeln(sout, 'Content-Type: text/plain;');
    WriteContentType(sout);
    writeln(sout, ' charset="iso-8859-1"');
    writeln(sout, 'Content-Transfer-Encoding: 7bit');
    writeln(sout); //must have a blank line here or it won't work
    writeln(sout, MessageText);

      // attachments : include them
    if length(TheAttachedFiles) > 0 then //if there are any attached files at all
    begin

      for iLoc:= 0 to length(TheAttachedFiles) - 1 do
      begin
        if fileexists(TheAttachedFiles[iLoc].Filename) = false then
        begin
          writeln       ('A file that was attempted to be attached: ' + TheAttachedFiles[iLoc].Filename + ', was not found.' + #13#10 +
                        'Emailing aborted.' + #13#10 +
                        'Remove all SetLength calls if you have not attached any files.' + #13#10 +
                        'If you are attaching files, make sure SetLength specifies the number of files attached.'); //ERROR REPORTING

          halt;
        end;

        // what type of attachment do we have
        if TheAttachedFiles[iLoc].Disposition = [] then //user didn't specify, so regular attachment
          TheDisposition:= 'attachment';
        if TheAttachedFiles[iLoc].Disposition = [RegularAttach] then
          TheDisposition:= 'attachment';
        if AttachedFiles[iLoc].Disposition = [InlineAttach] then
          TheDisposition:= 'inline';

        writeln(sout, '------=' + BoundaryDivider);
        writeln(sout, 'Content-Type: ' + TheAttachedFiles[iLoc].FileType + ';');
        writeln(sout, ' name="' + TheAttachedFiles[iLoc].FileName + '"');
        writeln(sout, 'Content-Transfer-Encoding: base64');
        writeln(sout, 'Content-Disposition: ' + TheDisposition + '; filename="' + TheAttachedFiles[iLoc].FileName + '"');
        writeln(sout);      //must have a blank line here or it won't work

        // write each file attachment's contents
        writeln(sout, PrepFileContents(TheAttachedFiles[iLoc].FileName));
        writeln(sout);
      end;

    end;
    writeln(sout, '------=' + BoundaryDivider);

    writeln(sout, '.'); //kill the running sendmail executable
    result:= true;
  end else
  begin
    writeln('AssignStream returned error -1'); //ERROR REPORTING
    result:= false;
  end;

  close(sin);
  close(sout);

end;


//  END OF PUBLIC PROCEDURES/FUNCTIONS
{------------------------------------------------------------------------------}



// testing function, for internal testing, debugging, and prototyping only
function TestTestSendMail: string;
var
  SIN,
  SOUT:text;
  Args: array of string;
begin
  result:= '';
  setlength(args, 1);
  Args[0]:= 'testtest@someplace.com';

  if AssignStream(SIN, SOUT, SENDMAIL_PATH, Args) <> -1 then
  begin
    writeln(sout, 'test test test sendmail_unix test test');
    //  readln(sin);
    writeln(sout, '.'); //end connection
  end else
    writeln('AssignStream returned error -1');   //ERROR REPORTING

  close(sin);
  close(sout);



end;




end.

