{
StrWrap1.pas

Some string functions to make getting data out of a text files easy, without
using stringlists.

  Sometimes you find yourself using StringList much too often, when in fact
  all you need is a simple string. This unit is essentially a wrapper for
  readln, assign, reset (annoying stuff you don't need, in simple situations).

  Without using a StringList.. Have you ever wanted to just..
   -load a COMPLETE file directly into a regular string var, without all the
    hassles of readln, assign, rewrite, writeln, reset, or
    stringlist.loadfromfile ?
   -grab the first line of a text file and ignore the rest. Extemely simple
    settings files can be built this way.
    (use an ini file later, when the app actually becomes large enough to need it)
   -grab a specific line N from a text file directly and precisely, without
    using stringlist.strings[0] or excessive readln hassles.
   -Load a file directly into an array of strings without all the hassles.
   -Load the first N lines directly into an array of strings, without loading
    entire file into memory.
   -Load the first N lines of a file directly into a string, without loading
    entire file into memory.
   -plus more

 Regards,
  Lars aka L505
  http://z505.com
  
 To do:
 ------
  -Optional error checking? Let the coder check if file exists first? Not sure
   if this would be a useful feature.
  -optimize for speed.
   Some functions could be optimized, but more importantly, they work, so who
   cares for now.
  -error check on number of lines a file contains. If the file contains less
   lines than specified, report error. Not sure if this would be useful.
  -maybe submit some functions to RTL or StrUtils if they are useful as generic
   string functions that people use often.
  -offer shortstring version.. ansi-string is used by default, but maybe some
   people want to use shortstrings. Small settings files may only have short
   lines inside them anyway, that people may already know are a fixed length.

}

unit StrWrap1;

{$mode objfpc}
{$H+} //force ansi strings on

interface

uses
  SysUtils;

{Global type...}
 type
  StrArray = array of string;
  LnArray = array of integer;

{Global functions...}
 function StrLoadFile(FileName:string): string;
 function StrLoadLns(NumOfLines:integer; FileName:string): String;
 function StrLoadRng(FromLine:integer; ToLine:integer; FileName:string):String;
 function GetLn1(FileName:string): string;
 function GetLnN(LineNumber:integer; FileName:string): string;
 function ArrayLoadFile(FileName:string): StrArray;
 function Array0LoadFile(FileName:string): StrArray;
 function ArrayLoadLns(NumOfLines:integer; FileName:string): StrArray;
 function Array0LoadLns(NumOfLines:integer; FileName:string): StrArray;
 function ArrayLoadRng(FromLine:integer; ToLine:integer; FileName:string): StrArray;
 function Array0LoadRng(FromLine:integer; ToLine:integer; FileName:string): StrArray;
 function StrLoadExact(LineNumsArray:LnArray; FileName:string): string;
 function ArrayLoadExact(LineNumsArray:LnArray; FileName:string): StrArray;
 function Array0LoadExact(LineNumsArray:LnArray; FileName:string): StrArray;

implementation

const
{$ifdef WIN32}
 CRLF: string = #13#10; //windows uses CRLF
{$else}
 CRLF: string = #10;   //linux uses LF. Other operating systems use???
{$endif}

{ Load an entire text file into a string, including carriage returns. The string
  ends with an extra carriage return }
function StrLoadFile(FileName:string): string;
var
  F: text;
  str1,Line: string;
begin
  result:= ''; //safety
  str1:= '';
  if FileExists(FileName) = false then
  begin
    result:=''; //returns nothing if file not found
    exit;
  end;
  Assign(F, FileName);
  Reset(F);
  while not Eof(F) do
  begin
    Readln(F,Line);
    str1:= str1+Line+CRLF;
  end;
  result:= str1;
  close(F); //close up file
end;{ Todo: get rid of extra carriage return that is appended. Will require an extra
            if statement, so may cause some slow down }


{ Load the first N lines of a file into a string directly. Without the entire
  file getting loaded into memory!
  Note: an extra carriage return is appended to the end}
function StrLoadLns(NumOfLines:integer; FileName:string): String;
var
  F: text;
  str1: string;
  Line: string;
  iLoc:integer;
  GotLines: boolean; //to signal that we have 'got' all the lines we need
begin
  result:= ''; //safety
  if FileExists(FileName) = false then
  begin
   result:=''; //returns nothing if file not found
   exit;
  end;
  GotLines:= false; //initialize as false
  Assign(F, FileName);
  Reset(F);
  iLoc:=0;
  while not ( Eof(F) ) and ( GotLines <> true ) do
  begin
    inc(iLoc);
    Readln(F,Line);
    str1:= str1+Line+CRLF;
    if iLoc = NumOfLines then  //we've 'gotten' all the lines we need
     GotLines:= true;          //so exit this while loop
  end;
  result:= str1;
  close(F); //close up file
end; { Todo: get rid of extra carriage return that is appended. Will require an
       extra if statement, so may cause some slow down }


{ Load certain range of lines from a text file into a string.
  Example: lines 10 to 15 only }
function StrLoadRng(FromLine:integer; ToLine:integer; FileName:string):String;
var
  F: text;
  str1: string;
  Line: string;
  iLoc:integer;
  GotLines: boolean; //to signal that we have 'got' all the lines we need
begin
  result:= ''; //safety
  if FileExists(FileName) = false then
  begin
   result:=''; //returns nothing if file not found
   exit;
  end;
  GotLines:= false; //initialize as false
  Assign(F, FileName);
  Reset(F);
  iLoc:=0;
  while not ( Eof(F) ) and ( GotLines <> true ) do
  begin
    inc(iLoc);
    Readln(F,Line);
    if iLoc >= FromLine then   //we are at the start of the specified range
     str1:= str1+Line+CRLF;
    if iLoc = ToLine then      //we've 'gotten' all the lines we need
     GotLines:= true;          //so exit this while loop
  end;
  result:= str1;
  close(F); //close up file
end;


{ Get the first line of text into a string, from a specified file }
function GetLn1(FileName:string): string;
var
 F: text;
 Line: string;
begin
  result:= ''; //safety
  if FileExists(FileName) = false then
  begin
    result:= ''; //returns nothing if file not found
    exit;
  end;
  Assign(F, FileName);
  Reset(F);
  Readln(F,Line); //read only the first line of file
  result:= Line;
  close(F); //close up file
end;


{ Get specified Nth line in a text file into a string }
function GetLnN(LineNumber:integer; FileName:string): string;
var
  F: text;
  iLoc:integer; //local incremental integer
  Line: string;
  GotLine: boolean;
begin
  result:= ''; //safety
  if FileExists(FileName) = false then
  begin
   result:=''; //returns nothing if file not found
   exit;
  end;
  GotLine:=false; //initialize as false
  Assign(F, FileName);
  Reset(F);
  iLoc:=0;
  while not Eof(f) and  (GotLine <> true ) do //only continue reading the lines of the file until we have GOT our line that we want
  begin
   inc(iLoc);
   Readln(F,Line); //todo: optimize.. this may be expensive, since this is inside a loop.
   if iLoc = LineNumber then //when we hit the specified line number, make the result equal that line number's text
   begin
    result:= Line;
    GotLine:= true; //exit this while loop once we arrive to 'line N'
   end;
  end;
  close(F); //close up file
end;


{ Load an entire text file into an array of strings. Each string in the array
  becomes each line of the text file, i.e. separated by carriage returns.
  Note: the array contents start at array[1], since we deal with line 1, not
  line 0, for clarity.
  note: must do your own error checking.. if file does not exist you will get
        a range error if you use a loop out of range bounds}
function ArrayLoadFile(FileName:string): StrArray;
var
  F: text;
  Line: string;
  iLoc:integer;
begin
  setlength(result,1);
  if FileExists(FileName) = false then
  begin
   result[0]:=''; //returns nothing if file not found
   exit;
  end;
  result[0]:=''; //safety
  Assign(F, FileName);
  Reset(F);
  iLoc:=0;
  while not Eof(F) do
  begin
    inc(iLoc);
    Readln(F,Line);
    setlength(result,iLoc+1);
    result[iLoc]:= Line;
  end;
  close(F); //close up file
end; { Todo: optimize, maybe less setlengths somehow, since setlength is in a loop.}

{ Same as ArrayLoadFile except array starts at [0] instead of [1]
  note: must do your own error checking.. if file does not exist you will get
        a range error if you use a loop out of range bounds}
function Array0LoadFile(FileName:string): StrArray;
var
  F: text;
  Line: string;
  iLoc:integer;
begin
  setlength(result,1);
  if FileExists(FileName) = false then
  begin
   result[0]:=''; //returns nothing if file not found
   exit;
  end;
  result[0]:=''; //safety
  Assign(F, FileName);
  Reset(F);
  iLoc:= -1;
  while not Eof(F) do
  begin
    inc(iLoc);
    Readln(F,Line);
    setlength(result,iLoc);
    result[iLoc]:= Line;
  end;
  close(F); //close up file
end; { Todo: optimize, maybe less setlengths somehow, since setlength is in a loop.}


{ Load the first N lines of a file into an array of strings. Each line becomes
  a string in the array. Example: load the first 15 lines of a certain file.
  Note: the array contents start at array[1], since we deal with line 1, not
  line 0, for clarity. }
function ArrayLoadLns(NumOfLines:integer; FileName:string): StrArray;
var
  F: text;
  Line: string;
  iLoc:integer;
  GotLines: boolean; //to signal that we have 'got' all the lines we need
begin
  setlength(result,1);
  if FileExists(FileName) = false then
  begin
   setlength(result,NumOfLines+1);  //safety.. makes sure the user doesn't crash the program due to a range error, if the file doesn't exist
   result[0]:=''; //returns nothing if file not found
   exit;
  end;
  result[0]:=''; //safety
  GotLines:= false; //initialize as false
  Assign(F, FileName);
  Reset(F);
  iLoc:=0;
  setlength(result,NumOfLines+1); //we know the exact length that the array will be
  while not ( Eof(F) ) and ( GotLines <> true ) do
  begin
    inc(iLoc);
    Readln(F,Line);
    result[iLoc]:= Line;
    if iLoc = NumOfLines then  //we've 'gotten' all the lines we need
     GotLines:= true;          //so exit this while loop
  end;
  close(F); //close up file
end;

{ Same as ArrayLoadLns, except start at array[0] instead of array[1]  }
function Array0LoadLns(NumOfLines:integer; FileName:string): StrArray;
var
  F: text;
  Line: string;
  iLoc:integer;
  GotLines: boolean; //to signal that we have 'got' all the lines we need
begin
  if FileExists(FileName) = false then
  begin
   setlength(result,NumOfLines); //safety.. makes sure the user doesn't crash the program due to a range error, if the file doesn't exist
   result[0]:=''; //returns nothing if file not found
   exit;
  end;
  setlength(result,1);
  result[0]:=''; //safety
  GotLines:= false; //initialize as false
  Assign(F, FileName);
  Reset(F);
  iLoc:= -1;
  setlength(result,NumOfLines); //we know the exact length that the array will be
  while not ( Eof(F) ) and ( GotLines <> true ) do
  begin
    inc(iLoc);
    Readln(F,Line);
    result[iLoc]:= Line;
    if iLoc = NumOfLines-1 then  //we've 'gotten' all the lines we need (0 based, so -1)
     GotLines:= true;            //so exit this while loop
  end;
  close(F); //close up file
end;


{ Load a range of lines into a string array. Example: lines 11 to 16 into
  array[1] to array[6]
  note: array starts at 1 not 0 }
function ArrayLoadRng(FromLine:integer; ToLine:integer; FileName:string): StrArray;
var
  F: text;
  Line: string;
  iLoc,
  iLoc2:integer;
  GotLines: boolean; //to signal that we have 'got' all the lines we need
begin
  setlength(result,1);
  if FileExists(FileName) = false then
  begin
    setlength(result,ToLine-FromLine + 2); //safety.. makes sure the user doesn't crash the program due to a range error, if the file doesn't exist
    result[0]:=''; //returns nothing if file not found
    exit;
  end;
  result[0]:=''; //safety
  GotLines:= false; //initialize as false
  Assign(F, FileName);
  Reset(F);
  iLoc:= 0; iLoc2:=1;
  setlength(result,(ToLine-FromLine)+2); //we know the exact length that the array will be  +2 because range 4 to 6 is three lines, not two. So +1,  and setlength is 0 based, so another +1.
  while not ( Eof(F) ) and ( GotLines <> true ) do
  begin
    inc(iLoc);      //current line we are at
    Readln(F,Line);
    if iLoc >= FromLine then //we are in the correct range
    begin
      result[iLoc2]:= Line;
      inc(iLoc2);
    end;
    if iLoc = ToLine then   //we've 'gotten' all the lines we need
     GotLines:= true;       //so exit this while loop
  end;
  close(F); //close up file
end;

{ Same as Array0LoadRng but array starts at [0] instead of [1] }
function Array0LoadRng(FromLine:integer; ToLine:integer; FileName:string): StrArray;
var
  F: text;
  Line: string;
  iLoc,
  iLoc2:integer;
  GotLines: boolean; //to signal that we have 'got' all the lines we need
begin
  setlength(result,1);
  if FileExists(FileName) = false then
  begin
    setlength(result,ToLine-FromLine + 1); //safety.. makes sure the user doesn't crash the program due to a range error, if the file doesn't exist
    result[0]:=''; //returns nothing if file not found
    exit;
  end;
  result[0]:=''; //safety
  GotLines:= false; //initialize as false
  Assign(F, FileName);
  Reset(F);
  iLoc:= -1; iLoc2:= 0;
  setlength(result,(ToLine-FromLine)+1); //we know the exact length that the array will be  +2 because range 4 to 6 is three lines, not two. So +1
  while not ( Eof(F) ) and ( GotLines <> true ) do
  begin
    inc(iLoc);      //current line we are at
    Readln(F,Line);
    if iLoc >= FromLine-1 then //we are in the correct range
    begin
      result[iLoc2]:= Line;
      inc(iLoc2);
    end;
    if iLoc = ToLine-1 then   //we've 'gotten' all the lines we need (0 based, so -1)
     GotLines:= true;         //so exit this while loop
  end;
  close(F); //close up file
end;


{ Load a only certain lines into a string var. Example: lines 11,13,21
  Note: appends an extra carriage return to end of string }
function StrLoadExact(LineNumsArray:LnArray; FileName:string): string;
begin
//LnArray
end;


{ Load a only certain lines into a string array. Example: lines 11,13,21,8,6
  into array[1] to array[3]
  note: array starts at 1 not 0
        -lines specified in LineNumsArray can be in any order.. but
         ArrayLoadExactC will be faster, if you specify 6,8,11,13,21 instead.
         This is because ArrayLoadExact has to sort your stuff for you first,
         but this may be convenient at times.}
function ArrayLoadExact(LineNumsArray:LnArray; FileName:string): StrArray;
var
  F: text;
  Line: string;
  iLoc,
  iLoc2,
  iLoc3:integer;
  GotLines: boolean; //to signal that we have 'got' all the lines we need
begin
  setlength(result,1);
  if FileExists(FileName) = false then
  begin
    result[0]:=''; //returns nothing if file not found
    exit;
  end;
  result[0]:=''; //safety
  GotLines:= false; //initialize as false
  Assign(F, FileName);
  Reset(F);
  iLoc:= 0; iLoc2:= 1; iLoc3:= 0;
  setlength(result,length(LineNumsArray)+1); //we know the exact length that the array will be
  while not ( Eof(F) ) and ( GotLines <> true ) do
  begin
    inc(iLoc);      //current line we are at
    Readln(F,Line);
   {if LineNumsArray[iLoc3] = iLoc then
    begin
      inc(iLoc3);
      result[iLoc3+1]:= Line;
    end;  }
    for iLoc3:= 0 to length(LineNumsArray)-1 do //optimize
    begin
     if LineNumsArray[iLoc3] = iLoc then //we are on a line specified
       result[iLoc3+1]:= Line;
    end;
    if iLoc = LineNumsArray[length(LineNumsArray)-1] then   //we've 'gotten' all the lines we need
      GotLines:= true;         //so exit this while loop
  end;
  close(F); //close up file
end;


{ Same as ArrayLoadExact except array starts at [0] instead of [1] }
function Array0LoadExact(LineNumsArray:LnArray; FileName:string): StrArray;
var
  F: text;
  Line: string;
  iLoc,
  iLoc2,
  iLoc3:integer;
  GotLines: boolean; //to signal that we have 'got' all the lines we need
begin
  setlength(result,1);
  if FileExists(FileName) = false then
  begin
    result[0]:=''; //returns nothing if file not found
    exit;
  end;
  result[0]:=''; //safety
  GotLines:= false; //initialize as false
  Assign(F, FileName);
  Reset(F);
  iLoc:= -1; iLoc2:= 0; iLoc3:= 0;
  setlength(result,length(LineNumsArray)); //we know the exact length that the array will be
  while not ( Eof(F) ) and ( GotLines <> true ) do
  begin
    inc(iLoc);      //current line we are at
    Readln(F,Line);
   {if LineNumsArray[iLoc3] = iLoc then
    begin
      inc(iLoc3);
      result[iLoc3]:= Line;
    end;  }
    for iLoc3:= 0 to length(LineNumsArray)-1 do
    begin
     if LineNumsArray[iLoc3] = iLoc then //we are on a line specified
       result[iLoc3]:= Line;
    end;
    if iLoc = LineNumsArray[length(LineNumsArray)-1] then   //we've 'gotten' all the lines we need
      GotLines:= true;         //so exit this while loop
  end;
  close(F); //close up file
end;



end.
